@@ -14,7 +14,7 @@ from types import TracebackType
 # TODO: replace Optinal with | None
 from typing import Optional
 
-from diopter.utils import run_cmd, save_to_tmp_file
+from diopter.utils import run_cmd
 
 
 class Language(Enum):
@@ -76,9 +76,6 @@ class SourceProgram:
             the source code
         language (Language):
             the program's language
-        available_macros (tuple[str,...]):
-            known macros that can be defined to affect the program's compilation (this
-            is not exhaustive, additional, e.g., system macros, might be available)
         defined_macros (tuple[str,...]):
             macros that will be defined when compiling this program
         include_paths (tuple[str,...]):
@@ -91,7 +88,6 @@ class SourceProgram:
 
     code: str
     language: Language
-    available_macros: tuple[str, ...] = tuple()
     defined_macros: tuple[str, ...] = tuple()
     include_paths: tuple[str, ...] = tuple()
     system_include_paths: tuple[str, ...] = tuple()
@@ -100,12 +96,9 @@ class SourceProgram:
     def __post_init__(self) -> None:
         """Sanity checks"""
 
-        # macros should not include "-D"
-        for macro in chain(self.defined_macros, self.available_macros):
+        for macro in self.defined_macros:
             assert not macro.strip().startswith("-D")
 
-        assert set(self.defined_macros) <= set(self.available_macros)
-
         # (system) include paths must actually by paths and not include a flag
         for path in self.include_paths:
             assert not path.strip().startswith("-I")
@@ -309,19 +302,19 @@ class CompileError(Exception):
         return CompileError(output)
 
 
-class TemporarySourceCodeFile:
-    def __init__(self, program: SourceProgram):
-        self.program = program
-        self.fd_code: Optional[int] = None
-        self.code_file: str | None = None
+class TemporaryFile:
+    def __init__(self, *, contents: str, suffix: str):
+        self.contents = contents
+        self.suffix = suffix
+        self.path: str | None = None
 
     def __enter__(self) -> Path:
-        self.fd_code, self.code_file = tempfile.mkstemp(
-            suffix=self.program.language.to_suffix()
-        )
-        with open(self.code_file, "w") as f:
-            f.write(self.program.code)
-        return Path(self.code_file).absolute()
+        fd, self.path = tempfile.mkstemp(suffix=self.suffix)
+        os.close(fd)
+        if self.contents:
+            with open(self.path, "w") as f:
+                f.write(self.contents)
+        return Path(self.path).absolute()
 
     def __exit__(
         self,
@@ -329,11 +322,8 @@ class TemporarySourceCodeFile:
         exc_value: Optional[BaseException],
         exc_traceback: Optional[TracebackType],
     ) -> None:
-        if self.code_file and self.fd_code:
-            os.remove(self.code_file)
-            os.close(self.fd_code)
-        else:
-            raise CompileError("Compiler context exited but was not entered")
+        if self.path and Path(self.path).exists():
+            os.remove(self.path)
 
 
 @dataclass(frozen=True, kw_only=True)
@@ -475,7 +465,9 @@ class CompilationSetting:
         timeout: Optional[int] = None,
     ) -> CompilationInfo:
 
-        with TemporarySourceCodeFile(program) as code_file:
+        with TemporaryFile(
+            contents=program.code, suffix=program.get_file_suffix()
+        ) as code_file:
             cmd = self.get_compilation_cmd((program, code_file), output, True) + list(
                 additional_flags
             )
@@ -527,14 +519,14 @@ class CompilationSetting:
             str:
                 assembly code
         """
-        with tempfile.NamedTemporaryFile(suffix=".s") as asm_file:
+        with TemporaryFile(contents="", suffix=".s") as asm_file:
             self.compile_program(
                 program,
-                CompilationOutput(Path(asm_file.name), CompilationOutputKind.Assembly),
+                CompilationOutput(asm_file, CompilationOutputKind.Assembly),
                 additional_flags,
                 timeout,
             )
-            with open(asm_file.name, "r") as f:
+            with open(str(asm_file), "r") as f:
                 return f.read()
 
     def compile_program_to_object(
@@ -636,9 +628,9 @@ def find_standard_include_paths(
         tuple[str,...]:
             the include paths
     """
-    with tempfile.NamedTemporaryFile(suffix=".c" if not cpp else ".cpp") as tf:
+    with TemporaryFile(contents="", suffix=".c" if not cpp else ".cpp") as tf:
         # run clang with verbose output on an empty temporary file
-        cmd = [str(clang.exe), tf.name, "-c", "-o/dev/null", "-v"]
+        cmd = [str(clang.exe), str(tf), "-c", "-o/dev/null", "-v"]
         result = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
         assert result.returncode == 0
 
@@ -687,7 +679,6 @@ class ClangTool:
     exe: Path
     standard_c_include_paths: tuple[str, ...]
     standard_cxx_include_paths: tuple[str, ...]
-    timeout_s: int = 8
 
     # TODO: @cache?
     @staticmethod
@@ -710,7 +701,11 @@ class ClangTool:
         )
 
     def run_on_program(
-        self, program: SourceProgram, tool_flags: list[str], mode: ClangToolMode
+        self,
+        program: SourceProgram,
+        tool_flags: list[str],
+        mode: ClangToolMode,
+        timeout: Optional[int] = None,
     ) -> ClangToolResult:
         """Run the clang tool on the input program
 
@@ -722,16 +717,17 @@ class ClangTool:
             mode (ClangToolMode):
                 whether to capture and return stdout & stderr,
                 return the modified source code, or do both
+           timeout (int | None): timeout in seconds
 
         Returns:
             ClangToolResult:
                 captured stdout/stderr and/or modified source code
 
         """
-        with tempfile.NamedTemporaryFile(suffix=program.get_file_suffix()) as tf:
-            with open(tf.name, "w") as f:
-                f.write(program.code)
 
+        with TemporaryFile(
+            contents=program.code, suffix=program.get_file_suffix()
+        ) as tf:
             match program.language:
                 case Language.C:
                     standard_include_paths = self.standard_c_include_paths
@@ -740,7 +736,7 @@ class ClangTool:
 
             cmd = list(
                 chain(
-                    (str(self.exe), tf.name),
+                    (str(self.exe), str(tf)),
                     tool_flags,
                     (f"--extra-arg=-isystem{path}" for path in standard_include_paths),
                     (f"--extra-arg={flag}" for flag in program.get_compilation_flags()),
@@ -750,7 +746,7 @@ class ClangTool:
             try:
                 result = run_cmd(
                     cmd,
-                    timeout=self.timeout_s,
+                    timeout=timeout,
                     additional_env={"TMPDIR": str(tempfile.gettempdir())},
                 )
             except subprocess.CalledProcessError as e:
@@ -764,12 +760,12 @@ class ClangTool:
                         stderr=result.stderr,
                     )
                 case ClangToolMode.READ_MODIFIED_FILE:
-                    with open(tf.name, "r") as f:
+                    with open(str(tf), "r") as f:
                         return ClangToolResult(
                             modified_source_code=f.read(), stdout=None, stderr=None
                         )
                 case ClangToolMode.CAPTURE_OUT_ERR_AND_READ_MODIFIED_FILED:
-                    with open(tf.name, "r") as f:
+                    with open(str(tf), "r") as f:
                         return ClangToolResult(
                             modified_source_code=f.read(),
                             stdout=result.stdout,
@@ -813,29 +809,32 @@ class CComp:
             bool:
                 was the check successful?
         """
-        tf = save_to_tmp_file(program.code, suffix=program.language.to_suffix())
-        cmd = (
-            [
-                str(self.exe),
-                str(tf.name),
-                "-interp",
-                "-fall",
-            ]
-            + [
-                f"-I{ipath}"
-                for ipath in chain(program.include_paths, program.system_include_paths)
-            ]
-            + [f"-D{macro}" for macro in program.defined_macros]
-        )
-        try:
-            run_cmd(
-                cmd,
-                additional_env={"TMPDIR": str(tempfile.gettempdir())},
-                timeout=timeout,
+        assert program.language == Language.C
+        with TemporaryFile(contents=program.code, suffix=".c") as tf:
+            cmd = (
+                [
+                    str(self.exe),
+                    str(tf.name),
+                    "-interp",
+                    "-fall",
+                ]
+                + [
+                    f"-I{ipath}"
+                    for ipath in chain(
+                        program.include_paths, program.system_include_paths
+                    )
+                ]
+                + [f"-D{macro}" for macro in program.defined_macros]
             )
-        except subprocess.CalledProcessError:
-            return False
-        return True
+            try:
+                run_cmd(
+                    cmd,
+                    additional_env={"TMPDIR": str(tempfile.gettempdir())},
+                    timeout=timeout,
+                )
+            except subprocess.CalledProcessError:
+                return False
+            return True
 
 
 def __compilation_setting_parser() -> argparse.ArgumentParser:
